@@ -4,7 +4,7 @@
 *  Based upon CGI/Ex/Validate.pm v0.94 from Perl                     *
 *  For instructions on usage, see perldoc of CGI::Ex::Validate       *
 ***----------------------------------------------------------------**/
-// $Revision: 1.15 $
+// $Revision: 1.20 $
 
 function Validate () {
   this.error             = vob_error;
@@ -89,7 +89,7 @@ function vob_validate (form, val_hash) {
       var field_val = fields[j];
       var field = field_val['field'];
       if (! field) return this.error("Missing field key in validation");
-      if (found[field]) return this.error('Duplicate order found for '+field+' in group order or fields');
+      // if (found[field]) return this.error('Duplicate order found for '+field+' in group order or fields');
       found[field] = 1;
     }
     
@@ -431,8 +431,8 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
         if (typeof(ref[j]) == 'function') {
           if (! value.match(ref[j])) this.add_error(errors, field, tests[i], field_val, ifs_match);
         } else {
-          if (! (m = ref[j].match('^(!\\s*|)m([^\\s\\w])(.*)\\2([eigsmx]*)$')))
-            return this.error("Not sure how to parse that match "+ref[j]);
+          if (! (m = ref[j].match('^\\s*(!\\s*|)m([^\\s\\w])(.*)\\2([eigsmx]*)\\s*$')))
+            return this.error("Not sure how to parse that match ("+ref[j]+")");
           var not = m[1];
           var pat = m[3];
           var opt = m[4];
@@ -487,6 +487,15 @@ function vob_validate_buddy (form, field, field_val, N_level, ifs_match) {
     for (var i = 0; i < tests.length; i ++)
       if (! this.check_type(value, field_val[tests[i]], field, form))
         this.add_error(errors, field, tests[i], field_val, ifs_match);
+
+    /// do custom_js type checks
+    // this will allow for a custom piece of javascript
+    // the js is evaluated and should return 1 for success
+    // or 0 for failure - the variables field, value, and field_val (the hash) are available
+    var tests = this.filter_types('custom_js',types);
+    for (var i = 0; i < tests.length; i ++)
+      if (! eval(field_val[tests[i]]))
+        this.add_error(errors, field, tests[i], field_val, ifs_match);
   }
   
   /// all done - time to return
@@ -553,12 +562,13 @@ function vob_check_type (value, type, field, form) {
     if (! value) return 0;
     if (value.match('[^\\d\\- ]') || value.length > 16 || value.length < 13) return;
     /// simple mod10 check
-    value.replace(new RegExp('[\\- ]','g'), '');
+    value = value.replace(new RegExp('[\\- ]','g'), '');
     var sum = 0;
     var swc = 0;
+
     for (var i = value.length - 1; i >= 0; i --) {
       if (++ swc > 2) swc = 1;
-      var y = digit.charAt(i) * swc;
+      var y = value.charAt(i) * swc;
       if (y > 9) y -= 9;
       sum += y;
     }
@@ -815,6 +825,9 @@ function eob_get_error_text (err) {
     var _type = field_val["type"+dig];
     return name + " did not match type "+_type+".";
     
+  } else if (type == 'custom_js') {
+    return name + " did not match custom_js"+dig+" check.";
+
   }
 
   return alert("Missing error on field "+field+" for type "+type+dig);
